/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry;

import org.wso2.registry.exceptions.RegistryException;

import java.util.Date;

/**
 * Represents comments and its metadata. Note that only the Comment.text field needs to be filled
 * when adding new comments. All other attributes are ignored and they are filled with appropriate
 * values for the current context. Therefore, when contructing an instance of this class outside
 * the Regisrty impl, it is recommended to use new Comment("my comment text") constructor.
 */
public class Comment extends ResourceImpl {

    /**
     * Path of the comment. Each comment has a path in the form /projects/esb/config.xml;comments:12
     *
     */
    private transient String commentPath;

    /**
     * Comment text. This may contain any string inclusing HTML segments.
     */
    private transient String text;

    /**
     * Username of the user who added this comment.
     */
    private transient String user;

    /**
     * Date and time at which this comment is added.
     */
    private transient Date time;

    /**
     * Path of the resource on which this comment is made.
     */
    private transient String resourcePath;

    public Comment() {
    }

    public Comment(String commentText) {
        this.text = commentText;
        time = new Date();
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    /**
     * Get the comment time.
     * @return the time the comment was made.
     * @deprecated please use getCreatedTime() instead
     */
    public Date getTime() {
        return time;
    }

    public void setTime(Date time) {
        this.time = time;
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public Object getContent() throws RegistryException {
        return getText();
    }

    public String getDescription() {
        return getText();
    }

    public String getCommentPath() {
        return commentPath;
    }

    public void setCommentPath(String commentPath) {
        this.commentPath = commentPath;
    }

    public String getMediaType() {
        return "application/atom+xml";
    }

    public String getAuthorUserName() {
        return user;
    }

    public Date getLastModified() {
        return time;
    }

    public Date getCreatedTime() {
        return time;
    }
}
