/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry;

import org.wso2.registry.jdbc.dao.LogsDAO;
import org.wso2.registry.exceptions.RegistryException;

import javax.sql.DataSource;
import java.util.Date;

/**
 * The main purpose of this class is to handle pagination for log entries. From the registry API it will returns
 * LogEntryCollection , and from the LogEntryCollection user can get all the logs or he can ask for log for a
 * given range
 */
public class LogEntryCollection {

    private int logCount;
    private DataSource dataSource;

    private String resourcePath;
    private int action;
    private String userName;
    private Date from;
    private Date to;
    private boolean recentFirst;


    public int getLogCount() {
        return logCount;
    }

    public void setLogCount(int logCount) {
        this.logCount = logCount;
    }

    public LogEntry[] getLogEntries() throws RegistryException {
        LogsDAO logsDAO = new LogsDAO();
        return logsDAO.getLogs(resourcePath,
                action,
                userName,
                from,
                to,
                recentFirst,
                dataSource);
    }

    public LogEntry[] getLogEntries(int start, int pageLen) throws RegistryException {
        LogsDAO logsDAO = new LogsDAO();
        return logsDAO.getLogs(resourcePath,
                action,
                userName,
                from,
                to,
                recentFirst,
                start,
                pageLen,
                dataSource);
    }

    public DataSource getDataSource() {
        return dataSource;
    }

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }


    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public void setAction(int action) {
        this.action = action;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public void setFrom(Date from) {
        this.from = from;
    }

    public void setTo(Date to) {
        this.to = to;
    }

    public void setRecentFirst(boolean recentFirst) {
        this.recentFirst = recentFirst;
    }
}
