/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.filecache;

public class FileData {

    /**
     * Number of active input streams using the file.
     */
    private long connections;

    /**
     * Number of active (non discarded) resources using this content.
     */
    private long activeResourceCount;

    /**
     * Time at which the connections become 0.
     */
    private long idleFrom;

    private String filename;

    public FileData(String filename) {
        this.filename = filename;
        connections = 0;
        idleFrom = -1; // -1 indicates that the file is not idle
    }

    public long getConnections() {
        return connections;
    }

    public void setConnections(long connections) {
        this.connections = connections;
    }

    public void incrementConnection() {
        connections++;
        idleFrom = -1;
    }

    public void decrementConnection() {
        connections--;

        if (connections <= 0 && activeResourceCount <= 0) {
            idleFrom = System.currentTimeMillis();
        }
    }

    public long getActiveResourceCount() {
        return activeResourceCount;
    }

    public void incrementActiveResource() {
        activeResourceCount++;
        idleFrom = -1;
    }

    public void decrementActiveResource() {
        activeResourceCount--;

        if (connections <= 0 && activeResourceCount <= 0) {
            idleFrom = System.currentTimeMillis();
        }
    }

    public long getIdleFrom() {
        return idleFrom;
    }

    public void setIdleFrom(long idleFrom) {
        this.idleFrom = idleFrom;
    }

    public String getFilename() {
        return filename;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }
}
