/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.filters;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.handlers.RequestContext;

/**
 * This is a built-in Filter implementation to match against the URL (path) of the resources. This
 * can match different URLs for different handler operations. URLs to match for necessary handler
 * operations can be given as regular expressions. If a URL for particular operation is not
 * specified, it will always evaluate to false.
 *
 * Handler authors can use this Filter in thier configurations if the filtering requirement is only
 * to match against URLs of the resources.
 */
public class URLMatcher extends Filter {

    /**
     * URLs to match against resource path for handler oprations. They should be in the form of
     * regular expressions.
     */
    private String getPattern;
    private String putPattern;
    private String importPattern;
    private String deletePattern;
    private String putChildPattern;
    private String importChildPattern;

    public boolean handleGet(RequestContext requestContext) throws RegistryException {
        return getPattern != null &&
                requestContext.getResourcePath().getCompletePath().matches(getPattern);
    }

    public boolean handlePut(RequestContext requestContext) throws RegistryException {
        return putPattern != null &&
                requestContext.getResourcePath().getCompletePath().matches(putPattern);
    }

    public boolean handleImportResource(RequestContext requestContext) throws RegistryException {
        return importPattern != null &&
                requestContext.getResourcePath().getCompletePath().matches(importPattern);
    }

    public boolean handleDelete(RequestContext requestContext) throws RegistryException {
        return deletePattern != null &&
                requestContext.getResourcePath().getCompletePath().matches(deletePattern);
    }

    public boolean handlePutChild(RequestContext requestContext) throws RegistryException {
        return putChildPattern != null &&
                requestContext.getResourcePath().getCompletePath().matches(putChildPattern);
    }

    public boolean handleImportChild(RequestContext requestContext) throws RegistryException {
        return importChildPattern != null &&
               requestContext.getResourcePath().getCompletePath().matches(importChildPattern);
    }

    public String getGetPattern() {
        return getPattern;
    }

    public void setGetPattern(String getPattern) {
        this.getPattern = getPattern;
    }

    public String getPutPattern() {
        return putPattern;
    }

    public void setPutPattern(String putPattern) {
        this.putPattern = putPattern;
    }

    public String getImportPattern() {
        return importPattern;
    }

    public void setImportPattern(String importPattern) {
        this.importPattern = importPattern;
    }

    public String getDeletePattern() {
        return deletePattern;
    }

    public void setDeletePattern(String deletePattern) {
        this.deletePattern = deletePattern;
    }

    public String getPutChildPattern() {
        return putChildPattern;
    }

    public void setPutChildPattern(String putChildPattern) {
        this.putChildPattern = putChildPattern;
    }

    public String getImportChildPattern() {
        return importChildPattern;
    }

    public void setImportChildPattern(String importChildPattern) {
        this.importChildPattern = importChildPattern;
    }
}
