/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.users.UserStoreAdmin;
import org.wso2.registry.users.UserStoreException;

import java.util.Map;

public class RegistryUserStoreAdmin implements UserStoreAdmin {

    private UserStoreAdmin coreUserStoreAdmin;

    public RegistryUserStoreAdmin(UserStoreAdmin userStoreAdmin) {
        coreUserStoreAdmin = userStoreAdmin;
    }

    //public RegistryUserStoreAdmin(DataSource dataSource) throws UserStoreException {
    //    super(dataSource);
    //}

    public void addUser(String userName, Object credential) throws UserStoreException {
        coreUserStoreAdmin.addUser(userName, credential);
        addUserToRole(userName, RegistryConstants.EVERYONE_ROLE);
    }

    public void deleteUser(String userName) throws UserStoreException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER) ||
            userName.equals(RegistryConstants.ANONYMOUS_USER)) {

            String msg = "Could not remove the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        coreUserStoreAdmin.deleteUser(userName);
    }


    public void deleteRole(String roleName) throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE) ||
            roleName.equals(RegistryConstants.EVERYONE_ROLE) ||
            roleName.equals(RegistryConstants.GUESTS_ROLE)) {

            String msg = "Could not remove the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        coreUserStoreAdmin.deleteRole(roleName);
    }


    public void removeUserFromRole(String userName, String roleName) throws UserStoreException {

        if (roleName.equals(RegistryConstants.EVERYONE_ROLE)) {

            String msg = "Could not remove the everyone role from the user: " +
                         userName + ". All users belong to the everyone role.";
            throw new UserStoreException(msg, false);
        }

        coreUserStoreAdmin.removeUserFromRole(userName, roleName);
    }


    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserStoreException {

        // we should not allow to edit anoymous or system users.
        if (RegistryConstants.SYSTEM_USER.equals(userName) || 
                RegistryConstants.ANONYMOUS_USER.equals(userName)) {

            String msg = "Could not edit the system defined user: " + userName;
            throw new UserStoreException(msg);
        }

        coreUserStoreAdmin.updateUser(userName, newCredential, oldCredential);
    }


    public void updateUser(String userName, Object newCredential) throws UserStoreException {
        coreUserStoreAdmin.updateUser(userName, newCredential);
    }

    public void setUserProperties(String userName, Map properties) throws UserStoreException {
        coreUserStoreAdmin.setUserProperties(userName, properties);
    }

    public void addRole(String roleName) throws UserStoreException {
        coreUserStoreAdmin.addRole(roleName);
    }

    public void setRoleProperties(String roleName, Map properties) throws UserStoreException {
        coreUserStoreAdmin.setRoleProperties(roleName, properties);
    }

    public void addUserToRole(String userName, String roleName) throws UserStoreException {
        coreUserStoreAdmin.addUserToRole(userName, roleName);
    }

    public String[] getAllUserNames() throws UserStoreException {
        return coreUserStoreAdmin.getAllUserNames();
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        return coreUserStoreAdmin.isExistingUser(userName);
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        return coreUserStoreAdmin.isExistingRole(roleName);
    }

    public Map getUserProperties(String userName) throws UserStoreException {
        return coreUserStoreAdmin.getUserProperties(userName);
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        return coreUserStoreAdmin.getUserPropertyNames();
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        return coreUserStoreAdmin.getUserNamesWithPropertyValue(propertyName, propetyValue);
    }

    public String[] getAllRoleNames() throws UserStoreException {
        return coreUserStoreAdmin.getAllRoleNames();
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {
        return coreUserStoreAdmin.getRoleProperties(roleName);
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        return coreUserStoreAdmin.getUserRoles(userName);
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        return coreUserStoreAdmin.getUsersInRole(roleName);
    }
}
