/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm.pathbased;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.jdbc.dao.ResourceDAO;
import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.users.UserStoreException;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;

public class PathBasedAccessControlAdmin implements AccessControlAdmin {

    private static final Log log = LogFactory.getLog(PathBasedAccessControlAdmin.class);

    private AccessControlAdmin coreACA;

    private DataSource dataSource;
    private ResourceDAO resourceDAO = new ResourceDAO();

    public PathBasedAccessControlAdmin(AccessControlAdmin coreACA, DataSource dataSource) {
        this.coreACA = coreACA;
        this.dataSource = dataSource;
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
        coreACA.clearUserAuthorization(userName, getResourceID(resourceId), action);
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        coreACA.denyUser(userName, getResourceID(resourceId), action);
    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {
        coreACA.clearRoleAuthorization(roleName, getResourceID(resourceId), action);
    }

    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        coreACA.denyRole(roleName, getResourceID(resourceId), action);
    }

    public void authorizeRole(String roleName, String resourceId, String action
    ) throws UserStoreException {
        coreACA.authorizeRole(roleName, getResourceID(resourceId), action);
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        coreACA.clearResourceAuthorizations(getResourceID(resourceId));
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
        coreACA.copyAuthorizations(getResourceID(fromResourceId), getResourceID(toResourceId));
    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        coreACA.authorizeUser(userName, getResourceID(resourceId), action);
    }

    public boolean isRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {
        return coreACA.isRoleAuthorized(roleName, getResourceID(resourceId), action);
    }

    public boolean isUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        return coreACA.isUserAuthorized(userName, getResourceID(resourceId), action);
    }

    public String[] getAllowedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getAllowedUsersForResource(getResourceID(resourceId), action);
    }

    public String[] getDeniedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getDeniedUsersForResource(getResourceID(resourceId), action);
    }

    public String[] getAllowedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getAllowedRolesForResource(getResourceID(resourceId), action);
    }

    public String[] getDeniedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getDeniedRolesForResource(getResourceID(resourceId), action);
    }

    private String getResourceID(String resourcePath) throws UserStoreException {

        String resourceID = null;

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
            resourceID = resourceDAO.getResourceID(resourcePath, conn);

        } catch (SQLException e) {

            String msg = "Failed to get resource ID of the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new UserStoreException(msg, e);

        } finally {

            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    String msg = "Failed to close the database connection used to get the " +
                            "resource ID of the resource " + resourcePath + ". " + e.getMessage();
                    log.error(msg, e);
                }
            }
        }

        return resourceID;
    }
}
