/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.servlet.utils;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.session.UserRegistry;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

public class Utils {

    /**
     * Extract the paramters from the query string. e.g. for the URL http://localhost:8080/registry/d1/myresource?action=tag,tag_value=support
     * project we have the query string "action=tag,tag_value=support project" it should be
     * extracted to map as key=action, value=tag key=tag_value, value=support project
     *
     * @param queryString query string of the URL
     * @return Map containg paramter name --> paramter value pairs
     */
    public static Map getParameters(String queryString) {
        Map paramMap = new HashMap();
        String[] params = queryString.split(",");
        for (int i = 0; i < params.length; i++) {
            String[] param = params[i].split("=");
            paramMap.put(param[0].trim(), param[1].trim());
        }
        return paramMap;
    }

    /**
     * Returns the registry associated with the current session. If a user registry is not found,
     * new SecureRegistry instance is created with annoymous user and associated for the current
     * session.
     *
     * @param request Servlet request
     * @return SecureRegistry instance for the current session.
     * @throws org.wso2.registry.exceptions.RegistryException
     *          : if something went wrong
     */
    public static synchronized UserRegistry getSecureRegistry(HttpServletRequest request)
            throws RegistryException {
        UserRegistry registry;
        Object o = request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);
        if (o != null) {
            registry = (UserRegistry)o;
        } else {
            EmbeddedRegistry embeddedRegistry = (EmbeddedRegistry)request.
                    getSession().getServletContext().getAttribute(RegistryConstants.REGISTRY);

            registry = embeddedRegistry.getUserRegistry();

            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }
        return registry;
    }

    public static boolean isLoggedIn(HttpServletRequest request) {

        UserRegistry userRegistry =
                (UserRegistry)request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        return userRegistry != null &&
               !userRegistry.getUserName().equals(RegistryConstants.ANONYMOUS_USER);
    }

    public static void logInUser(HttpServletRequest request, String userName, String password)
            throws RegistryException {

        ServletContext context = request.getSession().getServletContext();
        EmbeddedRegistry embeddedRegistry =
                (EmbeddedRegistry)context.getAttribute(RegistryConstants.REGISTRY);

        UserRegistry userRegistry = embeddedRegistry.getUserRegistry(userName, password);

        request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, userRegistry);
    }
}
