package org.wso2.registry.users.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.def.DefaultAccessControlAdmin;
import org.wso2.registry.users.def.DefaultRealmConfig;
import org.wso2.registry.users.def.util.DefaultDatabaseUtil;

public class HybridAccessControlAdmin extends HybridAuthorizer implements AccessControlAdmin{
    private static Log log = LogFactory.getLog(DefaultAccessControlAdmin.class);

    public HybridAccessControlAdmin(DataSource dataSource, String algo) {
        super(dataSource, algo);
    }

    public HybridAccessControlAdmin(DataSource dataSource) {
        super(dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST);
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getPermission = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_PERMISSION_SQL);
            getPermission.setString(1, resourceId);
            getPermission.setString(2, action);

            ResultSet rs = getPermission.executeQuery();
            int pid;
            if (rs.next()) {
                pid = rs.getInt(HybridRealmConstants.COLUMN_NAME_ID);
            } else {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }
            
            PreparedStatement clearUPStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_USER_PERMISSION_SQL);
            clearUPStmt.setString(1, userName);
            clearUPStmt.setInt(2, pid);
            clearUPStmt.executeUpdate();
            dbConnection.commit();
            getPermission.close();
            clearUPStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        short deny = 0;
        addAllowDenyUserPermission(userName, resourceId, action, deny);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        short allow = 1;
        addAllowDenyRolePermission(roleName, resourceId, action, allow);

    }

    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        short deny = 0;
        addAllowDenyRolePermission(roleName, resourceId, action, deny);
    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getPermission = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_PERMISSION_SQL);
            getPermission.setString(1, resourceId);
            getPermission.setString(2, action);

            ResultSet rs = getPermission.executeQuery();
            String pid;
            if (rs.next()) {
                pid = rs.getString(HybridRealmConstants.COLUMN_NAME_ID);
            } else {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }

            int rid = DefaultDatabaseUtil.getRoleId(dbConnection, roleName);

            PreparedStatement deleteRolesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_ROLE_PERMISSION_SQL);
            deleteRolesStmt.setInt(1, rid);
            deleteRolesStmt.setString(2, pid);
            deleteRolesStmt.executeUpdate();

            dbConnection.commit();
            getPermission.close();
            deleteRolesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement clearResourceAuthorizationsStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_PERMISSION_ON_RESOURCE_SQL);
            clearResourceAuthorizationsStmt.setString(1, resourceId);
            clearResourceAuthorizationsStmt.executeUpdate();
            dbConnection.commit();
            clearResourceAuthorizationsStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getResourcePermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_RESOURCE_PERMISSION_SQL);
            getResourcePermissionStmt.setString(1, toResourceId);
            ResultSet rs = getResourcePermissionStmt.executeQuery();
            if (rs.next()) {
                throw new UserStoreException(Messages
                        .getMessage("copyAuthorizationFailedPermissionExist"));
            }

            getResourcePermissionStmt.setString(1, fromResourceId);
            rs = getResourcePermissionStmt.executeQuery();
            PreparedStatement addPermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_PERMISSION_SQL);
            while (rs.next()) {
                String action = rs.getString(HybridRealmConstants.COLUMN_NAME_ACTION);
                String oldPermissionId = rs.getString(HybridRealmConstants.COLUMN_NAME_ID);

                addPermissionStmt.setString(1, toResourceId);
                addPermissionStmt.setString(2, action);
                addPermissionStmt.executeUpdate();

                // ***** read the id
                PreparedStatement getPermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.GET_PERMISSION_SQL);
                getPermissionStmt.setString(1, toResourceId);
                getPermissionStmt.setString(2, action);

                ResultSet rs2 = getPermissionStmt.executeQuery();

                int idPermission = -1;
                if (rs2.next()) {
                    idPermission = rs2.getInt(HybridRealmConstants.COLUMN_NAME_ID);
                }
                // ***** end read id

                PreparedStatement getRolePermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.GET_ROLE_PERMISSION_SQL);

                getRolePermissionStmt.setString(1, oldPermissionId);
                ResultSet rsr = getRolePermissionStmt.executeQuery();
                PreparedStatement addRolePermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.ADD_ROLE_PERMISSION_SQL);

                while (rsr.next()) {

                    String roleId = rsr.getString(HybridRealmConstants.COLUMN_NAME_ROLE_ID);
                    short allow = rsr.getShort(HybridRealmConstants.COLUMN_NAME_IS_ALLOWED);

                    addRolePermissionStmt.setInt(1, idPermission);
                    addRolePermissionStmt.setShort(2, allow);
                    addRolePermissionStmt.setString(3, roleId);
                    addRolePermissionStmt.executeUpdate();
                }

                PreparedStatement getUserPermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.GET_USER_PERMISSION_SQL);

                getUserPermissionStmt.setString(1, oldPermissionId);
                ResultSet rsu = getUserPermissionStmt.executeQuery();

                PreparedStatement addUserPermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.ADD_USER_PERMISSION_SQL);

                while (rsu.next()) {
                    String userId = rsu.getString(HybridRealmConstants.COLUMN_NAME_USER_ID);
                    short allow = rsu.getShort(HybridRealmConstants.COLUMN_NAME_IS_ALLOWED);

                    addUserPermissionStmt.setInt(1, idPermission);
                    addUserPermissionStmt.setShort(2, allow);
                    addUserPermissionStmt.setString(3, userId);
                    addUserPermissionStmt.executeUpdate();
                }

                getRolePermissionStmt.close();
                addRolePermissionStmt.close();
                getUserPermissionStmt.close();
                addUserPermissionStmt.close();

            }

            getResourcePermissionStmt.close();
            addPermissionStmt.close();

            dbConnection.commit();

        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorCopyingAuthorizations"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        short allow = 1;
        addAllowDenyUserPermission(userName, resourceId, action, allow);
    }

    /**
     * Gets the permission if it exists, else create the permission and returns
     * the id.
     * 
     * @return the id of the permission
     */
    protected int getOrAddPermissionId(Connection dbConnection, String resourceId, String action)
            throws UserStoreException {
        int permissionId = -1;
        try {
            PreparedStatement getPermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_PERMISSION_SQL);
            getPermissionStmt.setString(1, resourceId);
            getPermissionStmt.setString(2, action);

            ResultSet rs = getPermissionStmt.executeQuery();

            if (rs.next()) {
                permissionId = rs.getInt(HybridRealmConstants.COLUMN_NAME_ID);
            }

            PreparedStatement addPermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_PERMISSION_SQL);

            if (permissionId == -1) {
                // no permission so add it
                addPermissionStmt.setString(1, resourceId);
                addPermissionStmt.setString(2, action);
                addPermissionStmt.executeUpdate();
                dbConnection.commit();

                // now read the id again
                getPermissionStmt = dbConnection
                        .prepareStatement(HybridRealmConstants.GET_PERMISSION_SQL);
                getPermissionStmt.setString(1, resourceId);
                getPermissionStmt.setString(2, action);

                ResultSet rs2 = getPermissionStmt.executeQuery();

                if (rs2.next()) {
                    permissionId = rs2.getInt(HybridRealmConstants.COLUMN_NAME_ID);
                }

                dbConnection.commit();
            }

        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        }

        return permissionId;

    }

    protected void addAllowDenyUserPermission(String userName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;

        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

        

            int permissionId = this.getOrAddPermissionId(dbConnection, resourceId, action);

            PreparedStatement clearUPStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_USER_PERMISSION_SQL);
            clearUPStmt.setString(1, userName);
            clearUPStmt.setInt(2, permissionId);
            clearUPStmt.executeUpdate();

            PreparedStatement addUserPermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_USER_PERMISSION_SQL);
            addUserPermissionStmt.setInt(1, permissionId);
            addUserPermissionStmt.setShort(2, allow);
            addUserPermissionStmt.setString(3, userName);
            addUserPermissionStmt.executeUpdate();
            dbConnection.commit();
            addUserPermissionStmt.close();
            clearUPStmt.close();
        } catch (SQLException e) {
            e.printStackTrace();
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

    }

    protected void addAllowDenyRolePermission(String roleName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            int roleId = DefaultDatabaseUtil.getRoleId(dbConnection, roleName);

            if (roleId == -1) {
                throw new UserStoreException(Messages.getMessage("nullRole"));
            }

            int permissionId = this.getOrAddPermissionId(dbConnection, resourceId, action);

            PreparedStatement deleteRolesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_ROLE_PERMISSION_SQL);
            deleteRolesStmt.setInt(1, roleId);
            deleteRolesStmt.setInt(2, permissionId);
            deleteRolesStmt.executeUpdate();

            PreparedStatement addRolePermissionStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_ROLE_PERMISSION_SQL);
            addRolePermissionStmt.setInt(1, permissionId);
            addRolePermissionStmt.setShort(2, allow);
            addRolePermissionStmt.setInt(3, roleId);
            addRolePermissionStmt.executeUpdate();
            dbConnection.commit();
            addRolePermissionStmt.close();
            deleteRolesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }
}
