package org.wso2.registry.users.hybrid;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.authenticator.Authenticator;

import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreAdmin;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.UserStoreReader;
import org.wso2.registry.users.def.DefaultAccessControlAdmin;
import org.wso2.registry.users.def.DefaultAuthorizer;
import org.wso2.registry.users.def.DefaultRealmConfig;


public class HybridRealm implements UserRealm {

    private Authenticator authenticator = null;

    private Authorizer authorizer = null;
    private AccessControlAdmin aclAdmin = null;

    private UserStoreAdmin usAdmin = null;
    private UserStoreReader usReader = null;

    private DataSource dataSource = null;
    private HybridRealmConfig config = null;

    public void init(Object configBean) throws UserStoreException {

        if (!(configBean instanceof HybridRealmConfig)) {
            return;
        }
        HybridRealmConfig config = (HybridRealmConfig) configBean;

        BasicDataSource basicDataSource = (BasicDataSource)config.getDataSource();
        if (basicDataSource == null) {
            basicDataSource = new BasicDataSource();
            basicDataSource.setUrl(config.getConnectionURL());
            basicDataSource.setDriverClassName(config.getDriverName());
            basicDataSource.setUsername(config.getConnectionUserName());
            basicDataSource.setPassword(config.getConnectionPassword());
        }
        this.config = config;
        dataSource = (DataSource) basicDataSource;

        if(config.getAuthenticator() == null)
            throw new UserStoreException("authenticatorMustExist");
        
        authenticator = config.getAuthenticator();
        
        usAdmin = config.getUserStoreAdmin();
        usReader = config.getUserStoreReader();
        
        authorizer = new HybridAuthorizer(dataSource,
                DefaultRealmConfig.PERMISSION_BLOCK_FIRST);
        aclAdmin = new HybridAccessControlAdmin(dataSource);
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return aclAdmin;
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return authenticator;
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return authorizer;
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return this.usAdmin;
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return this.usReader;
    }

    public Object getRealmConfiguration() throws UserStoreException {
        HybridRealmConfig retConfig = null;
        if (config == null) {
            retConfig = new HybridRealmConfig();
        }
        return retConfig;
    }
}
