package org.wso2.registry.users.hybrid.ldap;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.UserStoreReader;
import org.wso2.registry.users.hybrid.HybridRealmConstants;


public class LDAPUserStoreReader implements UserStoreReader {

    protected DataSource dataSource = null;

    private static Log log = LogFactory.getLog(LDAPUserStoreReader.class);

    private DirContext context = null;

    private LDAPRealmConfig config = null;

    public LDAPUserStoreReader(LDAPRealmConfig config, DataSource dataSource, DirContext context) {
        this.config = config;
        this.context = context;
        this.dataSource = dataSource;
    }

    public String[] getAllUserNames() throws UserStoreException {
        try {
            NamingEnumeration answer = context.list(config.getUserContextName());
            List lst = new ArrayList();
            while (answer.hasMore()) {
                Object obj = answer.nextElement();
                lst.add(obj.toString());
            }

            String[] wholeNames = (String[]) lst.toArray(new String[lst.size()]);
            String[] names = new String[lst.size()];
            for (int i = 0; i < wholeNames.length; i++) {
                String[] temp1 = wholeNames[i].split(":");
                if (temp1.length > 0) {
                    String[] temp2 = temp1[0].split("=");
                    if (temp2.length > 1) {
                        names[i] = temp2[1].trim();
                    } else {
                        throw new UserStoreException("unknownUserNameFormatInLDAP");
                    }
                } else {
                    throw new UserStoreException("unknownUserNameFormatInLDAP");
                }
            }
            return names;
        } catch (NamingException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
    }

    public Map getUserProperties(String userName) throws UserStoreException {

        String[] attrIds = config.getAttributeIds().split(",");

        Map properties = new HashMap();

        try {

            Attributes results = null;
            // Use pattern or search for user entry
            if (config.getUserPattern() != null) {
                results = getGenericUserByPattern(userName, attrIds);
            }

            // else {
            // results = getGenericUserBySearch(userName, attrIds);
            // }

            if (results != null) {
                NamingEnumeration names = results.getAll();
                while (names.hasMoreElements()) {
                    Attribute attr = (Attribute) names.nextElement();
                    properties.put(attr.getID(), attr.get().toString());
                }
            }

        } catch (NamingException e) {
            log.debug("errorReadingFromUserStore");
            throw new UserStoreException("errorReadingFromUserStore", e);
        }

        return properties;

    }

    public String[] getUserPropertyNames() throws UserStoreException {
        String[] attrIds = config.getAttributeIds().split(",");
        return attrIds;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        boolean bFound = false;
        String[] names = this.getAllUserNames();
        for (String name : names) {
            if (name.equals(userName)) {
                bFound = true;
                break;
            }
        }
        return bFound;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }

    // default style
    public String[] getAllRoleNames() throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getAllRoleNamesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ROLES_ALL_SQL);
            ResultSet rs = getAllRoleNamesStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getAllRoleNamesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {

        Map props = new HashMap();
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getRolePropertiesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ROLE_ATTRIBUTES_SQL);
            getRolePropertiesStmt.setString(1, roleName);
            ResultSet rs = getRolePropertiesStmt.executeQuery();
            String colAttrName = HybridRealmConstants.COLUMN_NAME_ATTR_NAME;
            String colAttrValue = HybridRealmConstants.COLUMN_NAME_ATTR_VALUE;
            while (rs.next()) {
                String name = rs.getString(colAttrName);
                String value = rs.getString(colAttrValue);
                props.put(name, value);
            }
            getRolePropertiesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return props;

    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            names = this.getUserRoles(dbConnection, userName);
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getUserInRole = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_USERS_IN_ROLE_SQL);
            getUserInRole.setString(1, roleName);
            ResultSet rs = getUserInRole.executeQuery();
            String colName = HybridRealmConstants.COLUMN_NAME_USER_NAME;
            List lst = new ArrayList();
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            names = (String[]) lst.toArray(new String[lst.size()]);
            getUserInRole.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        boolean isExisting = false;
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            if (getRoleId(dbConnection, roleName) != -1) {
                isExisting = true;
            }
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

        return isExisting;
    }

    protected Attributes getGenericUserByPattern(String userName, String[] attrIds)
            throws NamingException {

        log.info("getGenericUserByPattern");

        if (userName == null)
            return (null);

        // Form the dn from the user pattern
        String dn = MessageFormat.format(config.getUserPattern(), new String[] { userName });

        // Get required attributes from user entry
        Attributes attrs = null;

        attrs = context.getAttributes(dn, attrIds);

        return attrs;

    }

    protected static String[] getUserRoles(Connection dbConnection, String userName)
            throws UserStoreException {
        String[] names = new String[0];
        try {
            PreparedStatement getUserRolesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_USER_ROLES_SQL);
            getUserRolesStmt.setString(1, userName);
            ResultSet rs = getUserRolesStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getUserRolesStmt.close();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return names;
    }

    
    
    public static int getRoleId(Connection dbConnection, String roleName) throws UserStoreException {
        int id = -1;
        try {
            PreparedStatement getRolIdStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ROLE_ID_SQL);
            getRolIdStmt.setString(1, roleName);
            ResultSet rs = getRolIdStmt.executeQuery();
            if (rs.next()) {
                id = rs.getInt(HybridRealmConstants.COLUMN_NAME_ID);
            }

            getRolIdStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return id;
    }
}
