/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.samples.handlers.servers;

import org.wso2.registry.jdbc.handlers.UIEnabledHandler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.Resource;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;

import java.util.Map;
import java.util.HashMap;

/**
 * This class is a part of server information management sample. This handler is engaged for server
 * collections (application/vnd.server-collection), which are collections of server information
 * resources (application/vnd.server.info).
 *
 * Server collections can only contain server info resources. This class also generates a table
 * view to display all child server info resource contents.
 */
public class ServerCollectionHandler extends UIEnabledHandler {

    public String[] getBrowseViews() {
        return new String[] {"table:Table view"};
    }

    public String[] getEditViews() {
        return new String[0];
    }

    public String[] getNewViews() {
        return new String[0];
    }

    public Resource getBrowseView(String viewName, RequestContext requestContext)
            throws RegistryException {

        String path = requestContext.getResourcePath().getPath();
        Resource resource = requestContext.getRepository().get(path);
        requestContext.setResource(resource);

        Collection c = (Collection) resource;

        String[] serverDetails = (String[]) c.getContent();

        StringBuffer htmlTable = new StringBuffer("<table border=1>" +
                "<tr><th>Server name</th><th>Description</th><th>IP Address</th><th>Ports</th><th>Adminstrator</th></tr>");

        for (int i = 0; i < serverDetails.length; i++) {
            String serverDetailPath = serverDetails[i];
            htmlTable.append(getContentAsTableRow(serverDetailPath, requestContext));
        }
        htmlTable.append("</table");

        resource.setContent(htmlTable.toString());
        return resource;
    }

    public Resource getNewView(String newViewKey, RequestContext requestContext)
            throws RegistryException {
        return null;
    }

    private String getContentAsTableRow(String path, RequestContext requestContext)
            throws RegistryException {

        Resource serverDetail = requestContext.getRepository().get(path);
        byte[] contentBytes = (byte[]) serverDetail.getContent();
        String content = new String(contentBytes);

        String[] fields = content.split("\n");

        Map<String, String> data = new HashMap();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        String name = data.get("name");
        String desc = data.get("description");
	if (desc.length() > 20) {
            desc = desc.substring(0, 20) + "...";
        }
        String ip = data.get("ip");
        String ports = data.get("ports");
        String admin = data.get("admin");

        String tableRow = "<tr><td><a href='/wso2registry/web" + path + ";view'>" + name + "</a></td><td>" + desc + "</td><td>" + ip +
                "</td><td>" + ports + "</td><td>" + admin + "</td></tr>";
        return tableRow;
    }

    public Resource getEditView(String editName, RequestContext requestContext)
            throws RegistryException {
        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {

        Resource childResource = requestContext.getResource();
        if (childResource != null) {
            String mediaType = childResource.getMediaType();
            if (!"application/vnd.server-info".equals(mediaType)) {
                String msg = "Invalid child media type. Server collections only allow server " +
                        "information files (application/vnd.server-info) as child resources.";
                throw new RegistryException(msg);
            }
        }
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }
}
