/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.samples.handlers.servers;

import org.wso2.registry.jdbc.handlers.UIEnabledHandler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;

import java.util.Map;
import java.util.HashMap;

/**
 * This class is a part of a sample to demonstrate the use of handler framework to model a server
 * information repository. Server details files (application/vnd.server-info) contain information
 * about individual servers. It is a set of name value pairs as shown below.
 *
 * name: HR mail server
 * description: Mail server of the HR department.
 * ip: 10.1.100.22
 * ports: 8090, 9000
 * admin: deep
 */
public class ServerDetailsHandler extends UIEnabledHandler {

    public String[] getBrowseViews() {
        return new String[] { "details:Details" };
    }

    public String[] getEditViews() {
        return new String[] { "edit:Edit" };
    }

    public String[] getNewViews() {
        return new String[] { "basic:Create basic server details file", "advanced:Type in server details (recommended only for advanced users)" };
    }

    public String getDefaultNewView() {
        return "basic";
    }

    public Resource getBrowseView(String viewName, RequestContext requestContext)
            throws RegistryException {

        String path = requestContext.getResourcePath().getPath();
        Resource resource = requestContext.getRepository().get(path);
        byte[] contentBytes = (byte[]) resource.getContent();

        String content = new String(contentBytes);
        String[] fields = content.split("\n");

        Map <String, String> data = new HashMap();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        String name = data.get("name");
        String desc = data.get("description");
        String ip = data.get("ip");
        String ports = data.get("ports");
        String admin = data.get("admin");

        String view = "<h3>Details of server " + name + "</h3>" +
                "<br/><br/>" + desc +
                "<br/><br/>IP Address: " + ip +
                "<br/>Ports: " + ports +
                "<br/>Administrator: " + admin;

        resource.setContent(view);
        return resource;
    }

    public Resource getEditView(String editName, RequestContext requestContext)
            throws RegistryException {

        String path = requestContext.getResourcePath().getPath();
        Resource resource = requestContext.getRepository().get(path);
        requestContext.setResource(resource);
        byte[] contentBytes = (byte[]) resource.getContent();

        String content = new String(contentBytes);
        String[] fields = content.split("\n");

        Map <String, String> data = new HashMap();
        for (int i = 0; i < fields.length; i++) {
            String field = fields[i];
            String[] parts = field.split(":");
            data.put(parts[0], parts[1]);
        }

        String name = data.get("name");
        String desc = data.get("description");
        String ip = data.get("ip");
        String ports = data.get("ports");
        String admin = data.get("admin");

        String view = "<form method='post' action='/wso2registry/custom'>" +
                "<input type='hidden' name='edit-processor' value='application/vnd.server-info'/>" +
                "<input type='hidden' name='view-type' value='edit'/>" +
                "<input type='hidden' name='resourcePath' value='"+ path + "'/>" +
                "<h3>Server name: <input type='text' name='server-name' value='" + name + "'/></h3>" +
                "<br/><br/>Description: <input type='text' name='server-desc' value='" + desc + "'/>" +
                "<br/><br/>IP Address: <input type='text' name='server-ip' value='" + ip + "'/>" +
                "<br/>Ports: <input type='text' name='server-ports' value='" + ports + "'/>" +
                "<br/>Administrator: <input type='text' name='server-admin' value='" + admin + "'/>" +
                "<input type='submit' value='Save server details'/>" +
                "</form>";

        resource.setContent(view);
        return resource;
    }

    public Resource getNewView(String newViewKey, RequestContext requestContext)
            throws RegistryException {

        ResourcePath resourcePath = requestContext.getResourcePath();
        String parentPath = resourcePath.getParameterValue("parentPath");

        String uiString;
        if ("basic".equals(newViewKey)) {

            uiString = "<form method='post' action='/wso2registry/custom'>" +
                    "<input type='hidden' name='edit-processor' value='application/vnd.server-info'/>" +
                    "<input type='hidden' name='view-type' value='new'/>" +
                    "<input type='hidden' name='view-key' value='basic'/>" +
                    "<input type='hidden' name='parentPath' value='"+ parentPath + "'/>" +
                    "Resource name: <input type='text' name='resourceName'/>" +
                    "<h3>Server name: <input type='text' name='server-name'/></h3>" +
                    "<br/><br/>Description: <input type='text' name='server-desc'/>" +
                    "<br/><br/>IP Address: <input type='text' name='server-ip' />" +
                    "<br/>Ports: <input type='text' name='server-ports'/>" +
                    "<br/>Administrator: <input type='text' name='server-admin'/>" +
                    "<input type='submit' value='Add new server details'/>" +
                    "</form>";

        } else {

            uiString = "<form method='post' action='/wso2registry/custom'>" +
                    "<input type='hidden' name='edit-processor' value='application/vnd.server-info'/>" +
                    "<input type='hidden' name='view-type' value='new'/>" +
                    "<input type='hidden' name='view-key' value='advanced'/>" +
                    "<input type='hidden' name='parentPath' value='"+ parentPath + "'/>" +
                    "Resource name: <input type='text' name='resourceName'/>" +
                    "<br/><br/><textarea name='details' cols='60' rows='15'></textarea>" +
                    "<input type='submit' value='Add new server details'/>" +
                    "</form>";
        }

        Resource resource = requestContext.getRegistry().newResource();
        resource.setMediaType("application/vnd.server-info");
        resource.setContent(uiString);
        return resource;
    }
}
