/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.samples.custom;

import org.wso2.registry.jdbc.handlers.EditProcessor;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.stream.XMLStreamException;
import java.io.StringWriter;

public class Axis2ModuleEditProcessor extends EditProcessor {

    private OMFactory fac;

    public boolean processEditContent(
            String path,
            String editViewName,
            HttpServletRequest request,
            HttpServletResponse response) throws RegistryException {

        String moduleName = request.getParameter("moduleName");
        String moduleClass = request.getParameter("moduleClass");

        fac = OMAbstractFactory.getOMFactory();
        OMElement moduleElement = fac.createOMElement("module", null);
        moduleElement.addAttribute("name", moduleName, null);
        moduleElement.addAttribute("class", moduleClass, null);

        moduleElement.addChild(getInflowElement(request));
        moduleElement.addChild(getOutflowElement(request));
        moduleElement.addChild(getInfaultflowElement(request));
        moduleElement.addChild(getOutfaultflowElement(request));

        StringWriter stringWriter = new StringWriter();

        try {
            moduleElement.serialize(stringWriter);
            stringWriter.flush();
        } catch (XMLStreamException e) {
            String msg = "Failed to serialize edited xml to a string. " + e.getMessage();
            throw new RegistryException(msg, e);
        }

        String contentString = stringWriter.getBuffer().toString();

        Registry registry = getRegistry(request);
        Resource resource = registry.get(path);
        resource.setContent(contentString);
        registry.put(path, resource);
        resource.discard();

        return false;
    }

    public boolean processNewContent(
            String path,
            String newViewKey,
            HttpServletRequest request,
            HttpServletResponse response) throws RegistryException {

        return false;
    }

    private OMElement getInflowElement(HttpServletRequest request) {

        String handlerName = request.getParameter("inflowHN");
        String handlerClass = request.getParameter("inflowHC");
        String phase = request.getParameter("inflowPhase");

        OMElement flowElement = fac.createOMElement("inflow", null);
        
        OMElement handlerElement = fac.createOMElement("handler", null);
        handlerElement.addAttribute("name", handlerName, null);
        handlerElement.addAttribute("class", handlerClass, null);
        flowElement.addChild(handlerElement);

        OMElement phaseElement = fac.createOMElement("order", null);
        phaseElement.addAttribute("phase", phase, null);
        handlerElement.addChild(phaseElement);

        return flowElement;
    }

    private OMElement getOutflowElement(HttpServletRequest request) {

        String handlerName = request.getParameter("outflowHN");
        String handlerClass = request.getParameter("outflowHC");
        String phase = request.getParameter("outflowPhase");

        OMElement flowElement = fac.createOMElement("outflow", null);

        OMElement handlerElement = fac.createOMElement("handler", null);
        handlerElement.addAttribute("name", handlerName, null);
        handlerElement.addAttribute("class", handlerClass, null);
        flowElement.addChild(handlerElement);

        OMElement phaseElement = fac.createOMElement("order", null);
        phaseElement.addAttribute("phase", phase, null);
        handlerElement.addChild(phaseElement);

        return flowElement;
    }

    private OMElement getInfaultflowElement(HttpServletRequest request) {

        String handlerName = request.getParameter("infflowHN");
        String handlerClass = request.getParameter("infflowHC");
        String phase = request.getParameter("infflowPhase");

        OMElement flowElement = fac.createOMElement("INfaultflow", null);

        OMElement handlerElement = fac.createOMElement("handler", null);
        handlerElement.addAttribute("name", handlerName, null);
        handlerElement.addAttribute("class", handlerClass, null);
        flowElement.addChild(handlerElement);

        OMElement phaseElement = fac.createOMElement("order", null);
        phaseElement.addAttribute("phase", phase, null);
        handlerElement.addChild(phaseElement);

        return flowElement;
    }

     private OMElement getOutfaultflowElement(HttpServletRequest request) {

        String handlerName = request.getParameter("outfflowHN");
        String handlerClass = request.getParameter("outfflowHC");
        String phase = request.getParameter("outfflowPhase");

        OMElement flowElement = fac.createOMElement("Outfaultflow", null);

        OMElement handlerElement = fac.createOMElement("handler", null);
        handlerElement.addAttribute("name", handlerName, null);
        handlerElement.addAttribute("class", handlerClass, null);
        flowElement.addChild(handlerElement);

        OMElement phaseElement = fac.createOMElement("order", null);
        phaseElement.addAttribute("phase", phase, null);
        handlerElement.addChild(phaseElement);

        return flowElement;
    }
}
