/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.samples.stats;

import org.wso2.registry.jdbc.handlers.EditProcessor;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;

/**
 * EditProcessor implementation of the ESBStatisticsHandler. This extracts the user inputs of
 * edit views provided by ESBStatisticsHandler and contructs a ESB statistics resource
 * content from them.
 */
public class ESBStatisticsEditProcessor extends EditProcessor {

    /**
     * This method extracts the user inputs from basic and details edit view and updates the ESB
     * statistics resource contents from them.
     *
     * @param path Path of the ESB statistics resource to update.
     * @param editViewKey UI key of the edit view.
     * @param request HttpServletRequest object containing the request details.
     * @param response HttpServletResponse object to be filled with response datails.
     * @throws RegistryException
     */
    public boolean processEditContent(
            String path,
            String editViewKey,
            HttpServletRequest request,
            HttpServletResponse response) throws RegistryException {

        if ("basic".equals(editViewKey)) {

            String serverName = request.getParameter("server-name");
            String serverPath = request.getParameter("server-path");

            Registry registry = getRegistry(request);
            Resource resource = registry.get(path);

            OMElement docElement;
            try {
                StAXOMBuilder builder = new StAXOMBuilder(resource.getContentStream());
                docElement = builder.getDocumentElement();
            } catch (Exception ae) {
                throw new RegistryException("Failed to parse the esb statistics file. " +
                        "All statistics should be in XML format.");
            }

            OMElement serverElement = docElement.getFirstChildWithName(new QName("server"));
            OMElement serverNameElement = serverElement.getFirstChildWithName(new QName("name"));
            serverNameElement.setText(serverName);

            OMElement serverPathElement = serverElement.getFirstChildWithName(new QName("path"));
            serverPathElement.setText(serverPath);

            docElement.build();

            resource.setContent(docElement.toString());
            registry.put(path, resource);
            resource.discard();

        } else if ("details".equals(editViewKey)) {

            String contentString = request.getParameter("stats");

            Registry registry = getRegistry(request);
            Resource resource = registry.get(path);
            resource.setContent(contentString);
            registry.put(path, resource);
            resource.discard();
        }

        return false;
    }

    public boolean processNewContent(String parentPath,
                                  String editViewName,
                                  HttpServletRequest request,
                                  HttpServletResponse response) throws RegistryException {

        String resourceName = request.getParameter("resourceName");
        String resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
        String content = request.getParameter("stats");

        Registry registry = getRegistry(request);
        Resource resource = registry.newResource();
        resource.setMediaType("application/vnd.esb-stats");
        resource.setContent(content);
        registry.put(resourcePath, resource);
        resource.discard();

        return false;
    }
}
