/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.samples.person;

import org.wso2.registry.jdbc.handlers.EditProcessor;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class PersonEditProcessor extends EditProcessor {

    public boolean processEditContent(String path, String editViewKey, HttpServletRequest request, HttpServletResponse response) throws RegistryException {

        if ("details".equals(editViewKey)) {

            String resourcePath = path;

            String personName = request.getParameter("personName");
            String fullName = request.getParameter("fullName");
            String mobilePhone = request.getParameter("mobilePhone");
            String homePhone = request.getParameter("homePhone");
            String email = request.getParameter("email");
            String address = request.getParameter("address");

            OMFactory fac = OMAbstractFactory.getOMFactory();
            OMElement personElement = fac.createOMElement("person", null);

            OMElement nameElement = fac.createOMElement("name", null);
            nameElement.setText(personName);
            personElement.addChild(nameElement);

            OMElement fullNameElement = fac.createOMElement("fullName", null);
            fullNameElement.setText(fullName);
            personElement.addChild(fullNameElement);

            OMElement mobileElement = fac.createOMElement("mobilePhone", null);
            mobileElement.setText(mobilePhone);
            personElement.addChild(mobileElement);

            OMElement homeElement = fac.createOMElement("homePhone", null);
            homeElement.setText(homePhone);
            personElement.addChild(homeElement);

            OMElement emailElement = fac.createOMElement("email", null);
            emailElement.setText(email);
            personElement.addChild(emailElement);

            OMElement addressElement = fac.createOMElement("address", null);
            addressElement.setText(address);
            personElement.addChild(addressElement);

            String content = personElement.toString();

            Registry registry = getRegistry(request);
            Resource resource = registry.get(resourcePath);
            resource.setContent(content.getBytes());
            registry.put(resourcePath, resource);
            resource.discard();

            return false;

        } else {

            return false;
        }
    }

    public boolean processNewContent(String parentPath, String newViewKey, HttpServletRequest request, HttpServletResponse response) throws RegistryException {

        if ("details".equals(newViewKey)) {

            String resourceName = request.getParameter("resourceName");
            String resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;

            String personName = request.getParameter("personName");
            String fullName = request.getParameter("fullName");
            String mobilePhone = request.getParameter("mobilePhone");
            String homePhone = request.getParameter("homePhone");
            String email = request.getParameter("email");
            String address = request.getParameter("address");

            OMFactory fac = OMAbstractFactory.getOMFactory();
            OMElement personElement = fac.createOMElement("person", null);

            OMElement nameElement = fac.createOMElement("name", null);
            nameElement.setText(personName);
            personElement.addChild(nameElement);

            OMElement fullNameElement = fac.createOMElement("fullName", null);
            fullNameElement.setText(fullName);
            personElement.addChild(fullNameElement);

            OMElement mobileElement = fac.createOMElement("mobilePhone", null);
            mobileElement.setText(mobilePhone);
            personElement.addChild(mobileElement);

            OMElement homeElement = fac.createOMElement("homePhone", null);
            homeElement.setText(homePhone);
            personElement.addChild(homeElement);

            OMElement emailElement = fac.createOMElement("email", null);
            emailElement.setText(email);
            personElement.addChild(emailElement);

            OMElement addressElement = fac.createOMElement("address", null);
            addressElement.setText(address);
            personElement.addChild(addressElement);
            
            String content = personElement.toString();

            Registry registry = getRegistry(request);
            Resource resource = registry.newResource();
            resource.setMediaType("application/vnd.person");
            resource.setContent(content.getBytes());
            registry.put(resourcePath, resource);
            resource.discard();

            return false;

        } else {

            return false;
        }
    }
}
